@@ -54,12 +54,13 @@ static const gchar *gda_sqlite_provider_get_database (GdaServerProvider *provide
 static gboolean gda_sqlite_provider_change_database (GdaServerProvider *provider,
 						     GdaConnection *cnc,
 						     const gchar *name);
-static gboolean gda_sqlite_provider_create_database (GdaServerProvider *provider,
-						     GdaConnection *cnc,
-						     const gchar *name);
-static gboolean gda_sqlite_provider_drop_database (GdaServerProvider *provider,
-						   GdaConnection *cnc,
-						   const gchar *name);
+static gchar *gda_sqlite_provider_get_specs (GdaServerProvider *provider, GdaClientSpecsType type);
+static gboolean gda_sqlite_provider_perform_action_params (GdaServerProvider *provider,
+							   GdaParameterList *params, 
+							   GdaClientSpecsType type, GError **error);
+static gboolean gda_sqlite_provider_drop_database_cnc (GdaServerProvider *provider,
+						       GdaConnection *cnc,
+						       const gchar *name);
 
 static GList *gda_sqlite_provider_execute_command (GdaServerProvider *provider,
 						   GdaConnection *cnc,
@@ -110,8 +111,9 @@ gda_sqlite_provider_class_init (GdaSqliteProviderClass *klass)
 	provider_class->get_server_version = gda_sqlite_provider_get_server_version;
 	provider_class->get_database = gda_sqlite_provider_get_database;
 	provider_class->change_database = gda_sqlite_provider_change_database;
-	provider_class->create_database = gda_sqlite_provider_create_database;
-	provider_class->drop_database = gda_sqlite_provider_drop_database;
+	provider_class->get_specs = gda_sqlite_provider_get_specs;
+	provider_class->perform_action_params = gda_sqlite_provider_perform_action_params;
+	provider_class->drop_database_cnc = gda_sqlite_provider_drop_database_cnc;
 	provider_class->execute_command = gda_sqlite_provider_execute_command;
 	provider_class->begin_transaction = gda_sqlite_provider_begin_transaction;
 	provider_class->commit_transaction = gda_sqlite_provider_commit_transaction;
@@ -196,7 +198,7 @@ gda_sqlite_provider_open_connection (GdaServerProvider *provider,
 	t_filename = gda_quark_list_find (params, "URI");
 
 	if (!t_filename || *t_filename != '/') {
-		gda_connection_add_error_string (cnc,
+		gda_connection_add_event_string (cnc,
 						 _("A full path must be specified on the "
 						   "connection string to open a database."));
 		return FALSE;
@@ -209,7 +211,7 @@ gda_sqlite_provider_open_connection (GdaServerProvider *provider,
 
 	if (errmsg != SQLITE_OK) {
 		printf("error %s", sqlite3_errmsg (scnc->connection));
-		gda_connection_add_error_string (cnc, sqlite3_errmsg (scnc->connection));
+		gda_connection_add_event_string (cnc, sqlite3_errmsg (scnc->connection));
 		sqlite3_close (scnc->connection);
 		g_free (scnc->file);
 		g_free (scnc);
@@ -221,7 +223,7 @@ gda_sqlite_provider_open_connection (GdaServerProvider *provider,
 
 	/* set SQLite library options */
 	if (!gda_sqlite_provider_single_command (sqlite_prv, cnc, "PRAGMA empty_result_callbacks = ON"))
-		gda_connection_add_error_string (cnc, _("Could not set empty_result_callbacks SQLite option"));
+		gda_connection_add_event_string (cnc, _("Could not set empty_result_callbacks SQLite option"));
 
 	/* make sure the internals are completely initialized now */
 	{
@@ -243,7 +245,7 @@ gda_sqlite_provider_open_connection (GdaServerProvider *provider,
 		}
 		else {
 			g_print ("error: %s", errmsg);
-			gda_connection_add_error_string (cnc, errmsg);
+			gda_connection_add_event_string (cnc, errmsg);
 			sqlite3_free (errmsg);
 			sqlite3_close (scnc->connection);
 			g_free (scnc->file);
@@ -282,7 +284,7 @@ gda_sqlite_provider_close_connection (GdaServerProvider *provider,
 
 	scnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_SQLITE_HANDLE);
 	if (!scnc) {
-		gda_connection_add_error_string (cnc, _("Invalid SQLite handle"));
+		gda_connection_add_event_string (cnc, _("Invalid SQLite handle"));
 		return FALSE;
 	}
 
@@ -304,7 +306,7 @@ sql_split (const gchar *sql)
 
 	while ((next = (const gchar *) strchr (next, ';')) != NULL) {
 		gchar *tmp = g_strndup (remainder, next - remainder + 1);
-		if (sqlite_complete (tmp)) {
+		if (sqlite3_complete (tmp)) {
 			string_list = g_slist_prepend (string_list, tmp);
 			n++;
 			remainder = (const gchar *) (next + 1);
@@ -340,7 +342,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc,
 
 	scnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_SQLITE_HANDLE);
 	if (!scnc) {
-		gda_connection_add_error_string (cnc, _("Invalid SQLITE handle"));
+		gda_connection_add_event_string (cnc, _("Invalid SQLITE handle"));
 		return NULL;
 	}
 
@@ -353,8 +355,10 @@ process_sql_commands (GList *reclist, GdaConnection *cnc,
 			SQLITEresult *sres;
 			GdaDataModel *recset;
 			gint status, i;
+			int changes;
 
 			sres = g_new0 (SQLITEresult, 1);
+			changes = sqlite3_total_changes (scnc->connection);
 			status = sqlite3_get_table (scnc->connection, arr[n],
 						   &sres->data,
 						   &sres->nrows,
@@ -362,20 +366,59 @@ process_sql_commands (GList *reclist, GdaConnection *cnc,
 						   &errmsg);
 			if (options & GDA_COMMAND_OPTION_IGNORE_ERRORS ||
 			    status == SQLITE_OK) {
+				gchar *tststr;
 
-				recset = gda_sqlite_recordset_new (cnc, sres);
-				if (GDA_IS_DATA_MODEL (recset)) {
-					gda_data_model_set_command_text (recset, arr[n]);
-					gda_data_model_set_command_type (recset, GDA_COMMAND_TYPE_SQL);
-					for (i = sres->ncols; i >= 0; i--)
-						gda_data_model_set_column_title (recset, i,
-								sres->data[i]);
-					reclist = g_list_append (reclist, recset);
+				g_strchug (arr[n]);
+				tststr = arr[n];
+				if (! g_ascii_strncasecmp (tststr, "SELECT", 6) ||
+				    ! g_ascii_strncasecmp (tststr, "EXPLAIN", 7)) {
+					recset = gda_sqlite_recordset_new (cnc, sres);
+					if (GDA_IS_DATA_MODEL (recset)) {
+						gda_data_model_set_command_text (recset, arr[n]);
+						gda_data_model_set_command_type (recset, GDA_COMMAND_TYPE_SQL);
+						for (i = sres->ncols; i >= 0; i--)
+							gda_data_model_set_column_title (recset, i,
+											 sres->data[i]);
+						reclist = g_list_append (reclist, recset);
+					}
 				}
-			} else {
-				GdaError *error = gda_error_new ();
-				gda_error_set_description (error, errmsg);
-				gda_connection_add_error (cnc, error);
+				else {
+					int newchanges;
+					GdaConnectionEvent *event;
+					gchar *str, *tmp, *ptr;
+
+					/* don't return a data model */
+					reclist = g_list_append (reclist, NULL);
+					
+					/* generate a notice about changes */
+					newchanges = sqlite3_total_changes (scnc->connection);
+					event = gda_connection_event_new (GDA_CONNECTION_EVENT_NOTICE);
+					ptr = tststr;
+					while (*ptr && (*ptr != ' ') && (*ptr != '\t') &&
+					       (*ptr != '\n'))
+						ptr++;
+					*ptr = 0;
+					tmp = g_ascii_strup (tststr, -1);
+					if (!strcmp (tmp, "DELETE"))
+						str = g_strdup_printf ("%s %d (see SQLite documentation for a \"DELETE * FROM table\" query)", 
+								       tmp, (newchanges - changes));
+					else {
+						if (!strcmp (tmp, "INSERT"))
+							str = g_strdup_printf ("%s %lld %d", tmp, 
+									       sqlite3_last_insert_rowid (scnc->connection),
+									       (newchanges - changes));
+						else
+							str = g_strdup_printf ("%s %d", tmp, (newchanges - changes));
+					}
+					gda_connection_event_set_description (event, str);
+					g_free (str);
+					gda_connection_add_event (cnc, event);
+				}
+			} 
+			else {
+				GdaConnectionEvent *error = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+				gda_connection_event_set_description (error, errmsg);
+				gda_connection_add_event (cnc, error);
 
 				g_list_foreach (reclist, (GFunc) g_object_unref, NULL);
 				g_list_free (reclist);
@@ -419,7 +462,7 @@ gda_sqlite_provider_get_database (GdaServerProvider *provider,
 	scnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_SQLITE_HANDLE);
 
 	if (!scnc) {
-		gda_connection_add_error_string (cnc, _("Invalid SQLite handle"));
+		gda_connection_add_event_string (cnc, _("Invalid SQLite handle"));
 		return NULL;
 	}
 
@@ -437,36 +480,96 @@ gda_sqlite_provider_change_database (GdaServerProvider *provider,
 	g_return_val_if_fail (GDA_IS_SQLITE_PROVIDER (sqlite_prv), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 
-	gda_connection_add_error_string (cnc, _("Only one database per connection is allowed"));
+	gda_connection_add_event_string (cnc, _("Only one database per connection is allowed"));
 	return FALSE;
 }
-	
+
+/* get_specs handler for the GdaSqliteProvider class */
+static gchar *
+gda_sqlite_provider_get_specs (GdaServerProvider *provider, GdaClientSpecsType type)
+{
+	gchar *specs, *file;
+        gint len;
+	GdaSqliteProvider *sqlite_prv = (GdaSqliteProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_SQLITE_PROVIDER (sqlite_prv), FALSE);
+
+	switch (type) {
+	case GDA_CLIENT_SPECS_CREATE_DATABASE:
+		file = g_build_filename (LIBGDA_DATA_DIR, "sqlite_specs_create_db.xml", NULL);
+		if (g_file_get_contents (file, &specs, &len, NULL))
+			return specs;
+		else
+			return NULL;
+		break;
+	default:
+		return NULL;
+	}
+}
+
 /* create_database handler for the GdaSqliteProvider class */
 static gboolean
-gda_sqlite_provider_create_database (GdaServerProvider *provider,
-				     GdaConnection *cnc,
-				     const gchar *name)
+gda_sqlite_provider_perform_action_params (GdaServerProvider *provider,
+					   GdaParameterList *params, 
+					   GdaClientSpecsType type,
+					   GError **error)
 {
 	GdaSqliteProvider *sqlite_prv = (GdaSqliteProvider *) provider;
+	gboolean retval;
+	GdaParameter *param = NULL;
 
 	g_return_val_if_fail (GDA_IS_SQLITE_PROVIDER (sqlite_prv), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	
+	switch (type) {
+	case GDA_CLIENT_SPECS_CREATE_DATABASE:
+		if (params)
+			param = gda_parameter_list_find (params, "URI");
+		if (!param) {
+			g_set_error (error, 0, 0,
+				     _("Missing parameter 'URI'"));
+			retval = FALSE;
+		}
+		else {
+			SQLITEcnc *scnc;
+			gint errmsg;
+			gchar *filename;
+			
+			filename = gda_value_get_string (gda_parameter_get_value (param));
+			
+			scnc = g_new0 (SQLITEcnc, 1);
+			errmsg = sqlite3_open (filename, &scnc->connection);
+			
+			if (errmsg != SQLITE_OK) {
+				g_set_error (error, 0, 0,
+					     sqlite3_errmsg (scnc->connection));
+				retval = FALSE;
+			}
+			else
+				retval = TRUE;
+			sqlite3_close (scnc->connection);
+			g_free (scnc);
+		}
+		break;
+	default:
+		g_set_error (error, 0, 0,
+			     _("Method not handled by this provider"));
+		return FALSE;
+	}
 
-	gda_connection_add_error_string (cnc, _("Only one database per connection is allowed"));
-	return FALSE;
+	return retval;
 }
 
 static gboolean
-gda_sqlite_provider_drop_database (GdaServerProvider *provider,
-				   GdaConnection *cnc,
-				   const gchar *name)
+gda_sqlite_provider_drop_database_cnc (GdaServerProvider *provider,
+				       GdaConnection *cnc,
+				       const gchar *name)
 {
 	GdaSqliteProvider *sqlite_prv = (GdaSqliteProvider *) provider;
 
 	g_return_val_if_fail (GDA_IS_SQLITE_PROVIDER (sqlite_prv), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 
-	gda_connection_add_error_string (cnc, _("To remove a SQLite database you should remove the database file by hand"));
+	gda_connection_add_event_string (cnc, _("To remove a SQLite database you should remove the database file by hand"));
 	return FALSE;
 }
 
@@ -548,7 +651,7 @@ gda_sqlite_provider_begin_transaction (GdaServerProvider *provider,
 	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
 
 	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
-		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		gda_connection_add_event_string (cnc, _("Transactions are not supported in read-only mode"));
 		return FALSE;
 	}
 
@@ -630,7 +733,7 @@ gda_sqlite_provider_single_command (const GdaSqliteProvider *provider,
 	scnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_SQLITE_HANDLE);
 
 	if (!scnc) {
-		gda_connection_add_error_string (cnc, _("Invalid SQLite handle"));
+		gda_connection_add_event_string (cnc, _("Invalid SQLite handle"));
 		return FALSE;
 	}
 	
@@ -638,9 +741,9 @@ gda_sqlite_provider_single_command (const GdaSqliteProvider *provider,
 	if (status == SQLITE_OK)
 		result = TRUE;
 	else {
-		GdaError *error = gda_error_new ();
-		gda_error_set_description (error, errmsg);
-		gda_connection_add_error (cnc, error);
+		GdaConnectionEvent *error = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+		gda_connection_event_set_description (error, errmsg);
+		gda_connection_add_event (cnc, error);
 
 		result = FALSE;
 	}
@@ -683,7 +786,7 @@ add_type_row (GdaDataModelArray *recset, const gchar *name,
 	value_list = g_list_append (NULL, gda_value_new_string (name));
 	value_list = g_list_append (value_list, gda_value_new_string (owner));
 	value_list = g_list_append (value_list, gda_value_new_string (comments));
-	value_list = g_list_append (value_list, gda_value_new_type (type));
+	value_list = g_list_append (value_list, gda_value_new_gdatype (type));
 	value_list = g_list_append (value_list, gda_value_new_string (NULL));
 
 	gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
@@ -718,7 +821,7 @@ init_table_fields_recset (GdaConnection *cnc)
 {
 	GdaDataModelArray *recset;
 	gint i;
-	GdaSqliteColData cols[9] = {
+	GdaSqliteColData cols[] = {
 		{ N_("Field name")	, GDA_VALUE_TYPE_STRING  },
 		{ N_("Data type")	, GDA_VALUE_TYPE_STRING  },
 		{ N_("Size")		, GDA_VALUE_TYPE_INTEGER },
@@ -727,7 +830,8 @@ init_table_fields_recset (GdaConnection *cnc)
 		{ N_("Primary key?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("Unique index?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("References")	, GDA_VALUE_TYPE_STRING  },
-		{ N_("Default value")   , GDA_VALUE_TYPE_STRING  }
+		{ N_("Default value")   , GDA_VALUE_TYPE_STRING  },
+		{ N_("Extra attributes"), GDA_VALUE_TYPE_STRING  }		
 	};
 
 	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (sizeof cols / sizeof cols[0]));
@@ -750,7 +854,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 
 	scnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_SQLITE_HANDLE);
 	if (!scnc) {
-		gda_connection_add_error_string (cnc, _("Invalid SQLITE handle"));
+		gda_connection_add_event_string (cnc, _("Invalid SQLITE handle"));
 		return NULL;
 	}
 
@@ -776,12 +880,12 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	/* Fetch more information in memory */
 	if (SQLITE_VERSION_NUMBER >= 3000000) {
 		Table *table;
-		gint i;
+		int i;
 		GList *list = NULL;
 
 		table = sqlite3FindTable (scnc->connection, tblname, NULL); /* FIXME: database name as 3rd arg */
 		if (!table) {
-			gda_connection_add_error_string (cnc, _("Can't find table %s"), tblname);
+			gda_connection_add_event_string (cnc, _("Can't find table %s"), tblname);
 			return GDA_DATA_MODEL (recset);
 		}
 
@@ -834,7 +938,8 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 				while (index && !found) {
 					if (index->pTable == table) {
 						if ((index->nColumn == 1) && 
-						    (index->aiColumn == i) && index->autoIndex)
+						    (*(index->aiColumn) == i) && 
+						    index->autoIndex)
 							found = TRUE;
 					}
 					index = index->pNext;
@@ -881,6 +986,13 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 				value = gda_value_new_string ("");
 			rowlist = g_list_append (rowlist, value);
 
+			/* extra attributes */
+			if ((table->iPKey == i) && table->autoInc)
+				value = gda_value_new_string ("AUTO_INCREMENT");
+			else
+				value = gda_value_new_string ("");
+			rowlist = g_list_append (rowlist, value);
+
 			list = g_list_append (list, rowlist);
 		}
 		
@@ -898,7 +1010,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 			
 			fa = gda_data_model_describe_column (GDA_DATA_MODEL (reclist->data), i);
 			if (!fa) {
-				gda_connection_add_error_string (cnc, _("Could not retrieve information for field"));
+				gda_connection_add_event_string (cnc, _("Could not retrieve information for field"));
 				g_object_unref (G_OBJECT (recset));
 				recset = NULL;
 				break;
@@ -948,7 +1060,7 @@ get_tables (GdaConnection *cnc, GdaParameterList *params, gboolean views)
 
 	scnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_SQLITE_HANDLE);
 	if (!scnc) {
-		gda_connection_add_error_string (cnc, _("Invalid SQLite handle"));
+		gda_connection_add_event_string (cnc, _("Invalid SQLite handle"));
 		return NULL;
 	}
 
@@ -984,7 +1096,7 @@ get_types (GdaConnection *cnc, GdaParameterList *params)
 	SQLITEcnc *scnc;
 	scnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_SQLITE_HANDLE);
 	if (!scnc) {
-		gda_connection_add_error_string (cnc, _("Invalid SQLITE handle"));
+		gda_connection_add_event_string (cnc, _("Invalid SQLITE handle"));
 		return NULL;
 	}
 	g_assert ((scnc->connection->flags & SQLITE_Initialized) || scnc->connection->init.busy);
@@ -1113,7 +1225,7 @@ get_procs (GdaConnection *cnc, GdaParameterList *params, gboolean aggs)
 
 	scnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_SQLITE_HANDLE);
 	if (!scnc) {
-		gda_connection_add_error_string (cnc, _("Invalid SQLITE handle"));
+		gda_connection_add_event_string (cnc, _("Invalid SQLITE handle"));
 		return NULL;
 	}
 
